/***
*
*	Copyright (c) 1996-2001, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "soundent.h"
#include "gamerules.h"

enum mp5_e
{
	MP5_LONGIDLE = 0,
	MP5_IDLE1,
	MP5_LAUNCH,
	MP5_RELOAD,
	MP5_DEPLOY,
	MP5_FIRE1,
	MP5_FIRE2,
	MP5_FIRE3,
};



LINK_ENTITY_TO_CLASS( weapon_mp5, CMP5 );
LINK_ENTITY_TO_CLASS( weapon_9mmAR, CMP5 );




//=========================================================
//=========================================================
int CMP5::SecondaryAmmoIndex( void )
{
	return m_iSecondaryAmmoType;
}

void CMP5::Spawn( )
{
	pev->classname = MAKE_STRING("weapon_9mmAR"); // hack to allow for old names
	Precache( );
        SET_MODEL(ENT(pev), "models/null.mdl");//w_
	m_iId = WEAPON_MP5;

	m_iDefaultAmmo = MP5_DEFAULT_GIVE;

	FallInit();// get ready to fall down.
}


void CMP5::Precache( void )
{
        PRECACHE_MODEL("models/null.mdl"); //weapon model.

        //PRECACHE_MODEL("models/w_9mmARclip.mdl");

        PRECACHE_SOUND ("cannon.wav");// H to the K
        PRECACHE_SOUND ("weapons/357_cock1.wav");  //no more rockets
        PRECACHE_SOUND ("weapons/rocketfire1.wav"); //fire rocket
        PRECACHE_SOUND ("items/9mmclip1.wav");

	m_usMP5 = PRECACHE_EVENT( 1, "events/mp5.sc" );
      //  m_usMP52 = PRECACHE_EVENT( 1, "events/mp52.sc" );
}

int CMP5::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "9mm";
	p->iMaxAmmo1 = _9MM_MAX_CARRY;
	p->pszAmmo2 = "ARgrenades";
	p->iMaxAmmo2 = M203_GRENADE_MAX_CARRY;
	p->iMaxClip = MP5_MAX_CLIP;
	p->iSlot = 2;
	p->iPosition = 0;
	p->iFlags = 0;
	p->iId = m_iId = WEAPON_MP5;
	p->iWeight = MP5_WEIGHT;

	return 1;
}

int CMP5::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

BOOL CMP5::Deploy( )
{                                         //v               p
        return DefaultDeploy( "models/null.mdl", "models/null.mdl", MP5_DEPLOY, "mp5" );
        m_CanFireSecondary = TRUE;
}


void CMP5::PrimaryAttack()
{

        m_pPlayer->SetAnimation( PLAYER_ATTACK1 );
	m_pPlayer->pev->effects = (int)(m_pPlayer->pev->effects) | EF_MUZZLEFLASH;

        UTIL_MakeVectors(m_pPlayer->pev->v_angle);
        Vector vecDir;
        Vector vecAiming = m_pPlayer->GetAutoaimVector( AUTOAIM_5DEGREES );


        Vector vecSrc;
        vecSrc    = m_pPlayer->GetGunPosition( ) + (gpGlobals->v_right * 14);
        vecDir = m_pPlayer->FireBulletsPlayer( 1, vecSrc, vecAiming, VECTOR_CONE_3DEGREES, 8192, BULLET_PLAYER_MP5, 2, 0, m_pPlayer->pev, m_pPlayer->random_seed );

        vecSrc    = m_pPlayer->GetGunPosition( ) + (gpGlobals->v_right * -14);
        vecDir = m_pPlayer->FireBulletsPlayer( 1, vecSrc, vecAiming, VECTOR_CONE_3DEGREES, 8192, BULLET_PLAYER_MP5, 2, 0, m_pPlayer->pev, m_pPlayer->random_seed );


/*
#ifdef CLIENT_DLL
	if ( !bIsMultiplayer() )
#else
	if ( !g_pGameRules->IsMultiplayer() )
#endif
	{
		// optimized multiplayer. Widened to make it easier to hit a moving player
		vecDir = m_pPlayer->FireBulletsPlayer( 1, vecSrc, vecAiming, VECTOR_CONE_6DEGREES, 8192, BULLET_PLAYER_MP5, 2, 0, m_pPlayer->pev, m_pPlayer->random_seed );
	}
	else
	{
		// single player spread
		vecDir = m_pPlayer->FireBulletsPlayer( 1, vecSrc, vecAiming, VECTOR_CONE_3DEGREES, 8192, BULLET_PLAYER_MP5, 2, 0, m_pPlayer->pev, m_pPlayer->random_seed );
	}
  */

  int flags;
#if defined( CLIENT_WEAPONS )
	flags = FEV_NOTHOST;
#else
	flags = 0;
#endif

	PLAYBACK_EVENT_FULL( flags, m_pPlayer->edict(), m_usMP5, 0.0, (float *)&g_vecZero, (float *)&g_vecZero, vecDir.x, vecDir.y, 0, 0, 0, 0 );

        m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 0.14;

}



void CMP5::SecondaryAttack( void )
{
//bc
        if ( !(m_CanFireSecondary) )
                return;

        if (m_pPlayer->m_rgAmmo[m_iSecondaryAmmoType] <= 0)
	{
                EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/357_cock1.wav", 1.0, ATTN_NORM);
                m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 0.16;
		return;
	}


        m_CanFireSecondary = FALSE;

        EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/rocketfire1.wav", 1.0, ATTN_NORM);

        UTIL_MakeVectors(m_pPlayer->pev->v_angle);
        Vector vecThrow = gpGlobals->v_forward * 2000;


        //how I chose the distance between the two missiles:
        // the player hull is -16 to 16. Therefore, the distance b/w the
        // two missiles cannot exceed that, or else a missile fired near
        // a wall will immediately be fired "inside" the wall and
        // immediately detonate! Therefore I chose -14/14.
        Vector vecSrc;
        vecSrc = m_pPlayer->GetGunPosition( ) + (gpGlobals->v_right * 14) + (gpGlobals->v_forward * 8);
        CGrenade::ShootContact( m_pPlayer->pev, vecSrc, vecThrow);

        vecSrc = m_pPlayer->GetGunPosition( ) + (gpGlobals->v_right * -14) + (gpGlobals->v_forward * 8);
        CGrenade::ShootContact( m_pPlayer->pev, vecSrc, vecThrow);

        m_pPlayer->m_rgAmmo[m_iSecondaryAmmoType]--;

        m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 0.1;
        m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 0.1;
}

void CMP5::Reload( void )
{
}


void CMP5::WeaponIdle( void )
{
        m_CanFireSecondary = TRUE;
}



class CMP5AmmoClip : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
                SET_MODEL(ENT(pev), "models/null.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
                PRECACHE_MODEL ("models/null.mdl");
		PRECACHE_SOUND("items/9mmclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		int bResult = (pOther->GiveAmmo( AMMO_MP5CLIP_GIVE, "9mm", _9MM_MAX_CARRY) != -1);
		if (bResult)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/9mmclip1.wav", 1, ATTN_NORM);
		}
		return bResult;
	}
};
LINK_ENTITY_TO_CLASS( ammo_mp5clip, CMP5AmmoClip );
LINK_ENTITY_TO_CLASS( ammo_9mmAR, CMP5AmmoClip );



class CMP5Chainammo : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_chainammo.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_chainammo.mdl");
		PRECACHE_SOUND("items/9mmclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		int bResult = (pOther->GiveAmmo( AMMO_CHAINBOX_GIVE, "9mm", _9MM_MAX_CARRY) != -1);
		if (bResult)
		{
                        EMIT_SOUND(ENT(pev), CHAN_WEAPON, "items/9mmclip1.wav", 1, ATTN_NORM);
		}
		return bResult;
	}
};
LINK_ENTITY_TO_CLASS( ammo_9mmbox, CMP5Chainammo );


class CMP5AmmoGrenade : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
                SET_MODEL(ENT(pev), "models/w_nuke.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
                PRECACHE_MODEL ("models/w_nuke.mdl");
                PRECACHE_SOUND("lockload.wav");
	}
//bc
	BOOL AddAmmo( CBaseEntity *pOther )
	{ 
                int hResult = pOther->TakeHealth( gSkillData.healthkitCapacity, DMG_GENERIC );
		int bResult = (pOther->GiveAmmo( AMMO_M203BOX_GIVE, "ARgrenades", M203_GRENADE_MAX_CARRY ) != -1);

                if (bResult || hResult)
                {
                        EMIT_SOUND(ENT(pOther->pev), CHAN_VOICE, "lockload.wav", 1, ATTN_NORM);
                        bResult = 1;
                }

                return bResult;
	}
};
LINK_ENTITY_TO_CLASS( ammo_mp5grenades, CMP5AmmoGrenade );
LINK_ENTITY_TO_CLASS( ammo_ARgrenades, CMP5AmmoGrenade );
